#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file IPRouteViewer.py
 @brief This RTC displays IP routing table information.
 @date $Date$

 @author 株式会社 東日本計算センター

 プロジェクト方針決定後に記載

"""
import sys
import copy
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

# Import RTM module
import RTC
import OpenRTM_aist
import IPRouteIF
from tk_ip_route_viewer import TkIPRouteViewer, NODE, ROUTE, INFO, ALL

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

# Setup tk_viewer
tk_viewer = TkIPRouteViewer()

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
iprouteviewer_spec = ["implementation_id", "IPRouteViewer",
                      "type_name",         "IPRouteViewer",
                      "description",
                      "This RTC displays IP routing table information.",
                      "version",           "1.0",
                      "vendor",            "EAC",
                      "category",          "Network",
                      "activity_type",     "STATIC",
                      "max_instance",      "1",
                      "language",          "Python",
                      "lang_type",         "SCRIPT",
                      ""]
# </rtc-template>


##
# @class IPRouteViewer
# @brief This RTC displays IP routing table information.
#
# 別紙"機能仕様書_IPRouteViewerRTC"参照
#
# 別紙"機能仕様書_IPRouteViewerRTC"参照
#
# 別紙"機能仕様書_IPRouteViewerRTC"参照
#
# 別紙"機能仕様書_IPRouteViewerRTC"参照
#
#
class IPRouteViewer(OpenRTM_aist.DataFlowComponentBase):
    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_IPRouteListIn = IPRouteIF.TimedRouteList(RTC.Time(0, 0), 0, [])
        """
        """
        self._IPRouteListInIn = OpenRTM_aist.InPort("IPRouteListIn",
                                                    self._d_IPRouteListIn)

        self._init_variable()

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

    ##
    # RTC初期化
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers
        self.addInPort("IPRouteListIn", self._IPRouteListInIn)

        # Set OutPort buffers

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # データリスナー登録
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onActivated(self, ec_id):
        self._listener = DataListener(self._cb_rcv_route_data)
        listener_type = OpenRTM_aist.ConnectorDataListenerType.ON_RECEIVED

        self._IPRouteListInIn.addConnectorDataListener(
            listener_type, self._listener)

        return RTC.RTC_OK

        ##
        # データリスナー登録解除
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onDeactivated(self, ec_id):
        listener_type = OpenRTM_aist.ConnectorDataListenerType.ON_RECEIVED

        self._IPRouteListInIn.removeConnectorDataListener(
            listener_type, self._listener)

        self._init_variable()
        tk_viewer.init_viewer()

        return RTC.RTC_OK

        ##
        # IP経路テーブル情報受信チェック周期処理
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):
        self._check_missing_node()

        return RTC.RTC_OK

        ##
        # エラーログ出力
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onAborting(self, ec_id):
        logger.error("ERROR: detected an Aborting state")

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _init_variable(self):
        self._route_list = []
        self._route_list_bk = []
        self._listener = None

    def get_route_data(self, node_id):
        ret = None
        for data in self._route_list:
            if node_id == data.id:
                ret = data
                break

        if not ret:
            logger.error("invalid requested node_id: " + node_id)

        return ret

    def _check_missing_node(self):
        for i, (r1, r2) in enumerate(zip(self._route_list,
                                         self._route_list_bk)):
            if r1 == r2:
                if r1.data:
                    del r1.data[:]
                    tk_viewer.draw_gui(ALL, r1.id, missing=True)
                    logger.info("missing node(%d): %s", i, r1.id)

        self._route_list_bk = copy.deepcopy(self._route_list)

    def _cb_rcv_route_data(self, rcv_route_list):
        is_new = True
        for i, route in enumerate(self._route_list):
            if route.id == rcv_route_list.id:
                self._route_list[i] = copy.deepcopy(rcv_route_list)
                is_new = False

                if route.data == rcv_route_list.data:
                    # no route change
                    tk_viewer.draw_gui(INFO, route.id)
                elif route.data:
                    tk_viewer.draw_gui(ROUTE | INFO, route.id)
                    logger.info("changed the route data of node(%d): %s",
                                i, route.id)
                    logger.info(rcv_route_list)
                else:
                    tk_viewer.draw_gui(ALL, route.id, missing=False)
                    logger.info("found the missing node(%d): %s", i, route.id)
                    logger.info(rcv_route_list)

                break

        if is_new:
            self._route_list.append(rcv_route_list)
            logger.info("new node(%d): %s",
                        len(self._route_list), rcv_route_list.id)
            logger.info(rcv_route_list)

            tk_viewer.draw_gui(NODE, rcv_route_list.id)


class DataListener(OpenRTM_aist.ConnectorDataListenerT):
    def __init__(self, func):
        self._plugin_func = func
        logger.info("added DataListener")

    def __del__(self):
        logger.info("removed DataListener")

    def __call__(self, info, cdrdata):
        route = IPRouteIF.TimedRouteList(RTC.Time(0, 0), 0, [])
        route_list = OpenRTM_aist.ConnectorDataListenerT.__call__(self, info,
                                                                  cdrdata,
                                                                  route)

        self._plugin_func(route_list)


def IPRouteViewerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=iprouteviewer_spec)
    manager.registerFactory(profile,
                            IPRouteViewer,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    IPRouteViewerInit(manager)

    # Create a component
    comp = manager.createComponent("IPRouteViewer")

    tk_viewer.set_data_src_func(comp.get_route_data)


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    tk_viewer.mainloop()
    mgr.shutdown()

if __name__ == "__main__":
    main()
